@@ -15,6 +15,7 @@
 #include <dirent.h>
 #include <errno.h>
 
+#include "platform.h"
 #include "errmac.h"
 #include "zx.h"
 #include "zxid.h"
@@ -31,13 +32,16 @@ NO WARRANTY, not even implied warranties. Licensed under Apache License v2.0\n\
 See http://www.apache.org/licenses/LICENSE-2.0\n\
 Send well researched bug reports to the author. Home: zxid.org\n\
 \n\
-Usage: zxcot [options] [dir]\n\
+Usage: zxcot [options] [dir]         # Gives listing of metadata\n\
        zxcot -a [options] [dir] <meta.xml\n\
        zxcot -b [options] [dir] <epr.xml\n\
        curl https://site.com/metadata.xml | zxcot -a [options] [dir]\n\
        zxcot -g https://site.com/metadata.xml [options] [dir]\n\
+       zxcot -m [options] >meta.xml  # Generate our own metadata\n\
        zxcot -p https://site.com/metadata.xml\n\
   [dir]            CoT directory. Default /var/zxid/cot\n\
+  -c CONF          Optional configuration string (default -c PATH=/var/zxid/)\n\
+                   Most of the configuration is read from /var/zxid/zxid.conf\n\
   -a               Add metadata from stdin\n\
   -b               Register Web Service, add Service EPR from stdin\n\
   -bs              Register Web Service and Bootstrap, add Service EPR from stdin\n\
@@ -45,6 +49,7 @@ Usage: zxcot [options] [dir]\n\
   -g URL           Do HTTP(S) GET to URL and add as metadata (if compiled w/libcurl)\n\
   -n               Dryrun. Do not actually add the metadata. Instead print it to stdout.\n\
   -s               Swap columns, for easier sorting by URL\n\
+  -m               Output metadata of this installation (our own metadata)\n\
   -p ENTID         Print sha1 name corresponding to an entity ID.\n\
   -v               Verbose messages.\n\
   -q               Be extra quiet.\n\
@@ -61,6 +66,7 @@ int swap = 0;
 int addmd = 0;
 int regsvc = 0;
 int regbs = 0;
+int genmd = 0;
 int dryrun = 0;
 int inflate_flag = 2;  /* Auto */
 int verbose = 1;
@@ -70,11 +76,14 @@ char* entid = 0;
 char* cotdir  = ZXID_PATH ZXID_COT_DIR;
 char* dimddir = ZXID_PATH ZXID_DIMD_DIR;
 char* uiddir  = ZXID_PATH ZXID_UID_DIR;
+zxid_conf* cf = 0;
 
-/* Called by:  main x9 */
+/* Called by:  main x8, zxcall_main, zxcot_main */
 static void opt(int* argc, char*** argv, char*** env)
 {
   int len;
+  struct zx_str* ss;
+  
   if (*argc <= 1) return;
   
   while (1) {
@@ -106,7 +115,17 @@ static void opt(int* argc, char*** argv, char*** env)
 	continue;
       case '\0':
 	++regsvc;
-	uiddir  = ZXID_PATH "idp" ZXID_UID_DIR;
+	dimddir  = ZXID_PATH "idp" ZXID_DIMD_DIR;
+	continue;
+      }
+      break;
+
+    case 'c':
+      switch ((*argv)[0][2]) {
+      case '\0':
+	++(*argv); --(*argc);
+	if ((*argc) < 1) break;
+	zxid_parse_conf(cf, (*argv)[0]);
 	continue;
       }
       break;
@@ -116,18 +135,18 @@ static void opt(int* argc, char*** argv, char*** env)
       case '\0':
 	if ((*argc) < 4) break;
 	printf(
-"<wsa:EndpointReference xmlns:wsa=\"http://www.w3.org/2005/08/addressing\" "
+"<a:EndpointReference xmlns:a=\"http://www.w3.org/2005/08/addressing\" "
 "xmlns:wsu=\"http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-wssecurity-utility-1.0.xsd\" "
     "notOnOrAfter=\"2037-01-05T23:03:59.001Z\" "
     "wsu:Id=\"EPRID92lFPo3ZNEt_3rHtJFoU\">"
-  "<wsa:Address>%s</wsa:Address>"
-  "<wsa:Metadata>"
+  "<a:Address>%s</a:Address>"
+  "<a:Metadata>"
     "<di:Abstract xmlns:di=\"urn:liberty:disco:2006-08\">%s</di:Abstract>"
     "<sbf:Framework xmlns:sbf=\"urn:liberty:sb\" version=\"2.0\"></sbf:Framework>"
     "<di:ProviderID xmlns:di=\"urn:liberty:disco:2006-08\">%s</di:ProviderID>"
     "<di:ServiceType xmlns:di=\"urn:liberty:disco:2006-08\">%s</di:ServiceType>"
-  "</wsa:Metadata>"
-"</wsa:EndpointReference>", (*argv)[1], (*argv)[2], (*argv)[3], (*argv)[4]);
+  "</a:Metadata>"
+"</a:EndpointReference>", (*argv)[1], (*argv)[2], (*argv)[3], (*argv)[4]);
 	exit(0);
       }
       break;
@@ -148,6 +167,10 @@ static void opt(int* argc, char*** argv, char*** env)
       case '\0':
 	++zx_debug;
 	continue;
+      case 'c':
+	ss = zxid_show_conf(cf);
+	fprintf(stderr, "\n======== CONF ========\n%.*s\n^^^^^^^^ CONF ^^^^^^^^\n",ss->len,ss->s);
+	continue;
       }
       break;
 
@@ -169,6 +192,14 @@ static void opt(int* argc, char*** argv, char*** env)
       }
       break;
 
+    case 'm':
+      switch ((*argv)[0][2]) {
+      case '\0':
+	++genmd;
+	continue;
+      }
+      break;
+
     case 'n':
       switch ((*argv)[0][2]) {
       case '\0':
@@ -209,7 +240,7 @@ static void opt(int* argc, char*** argv, char*** env)
 
     } 
     /* fall thru means unrecognized flag */
-    D("HERE");
+    DD("HERE");
     if (*argc)
       fprintf(stderr, "Unrecognized flag `%s'\n", (*argv)[0]);
     fprintf(stderr, help);
@@ -229,10 +260,13 @@ static void opt(int* argc, char*** argv, char*** env)
   }
 }
 
+/* --------------- reg_svc --------------- */
+
 /*() IdP and Discovery. Register service metadata to /var/zxid/idpdimd/XX,
  * and possibly boostrap to /var/zxid/idpuid/.all/.bs/YY */
 
-static int zxid_reg_svc(struct zxid_conf* cf, int bs_reg, int dry_run, const char* ddimd, const char* duid)
+/* Called by:  zxcot_main */
+static int zxid_reg_svc(zxid_conf* cf, int bs_reg, int dry_run, const char* ddimd, const char* duid)
 {
   char sha1_name[28];
   char path[ZXID_MAX_BUF];
@@ -240,15 +274,17 @@ static int zxid_reg_svc(struct zxid_conf* cf, int bs_reg, int dry_run, const cha
   char* uiddir;
   int got, fd;
   struct zx_root_s* r;
-  struct zx_a_EndpointReference_s* epr;
+  zxid_epr* epr;
   struct zx_str* ss;
   
-  read_all_fd(0, buf, sizeof(buf)-1, &got);
+  read_all_fd(0, buf, sizeof(buf)-1, &got);  /* Read EPR */
   buf[got] = 0;
   p = buf;
   
+  LOCK(cf->ctx->mx, "cot reg_svc");
   zx_prepare_dec_ctx(cf->ctx, zx_ns_tab, buf, buf + got);
   r = zx_DEC_root(cf->ctx, 0, 1);
+  UNLOCK(cf->ctx->mx, "cot reg_svc");
   if (!r || !r->EndpointReference) {
     ERR("Failed to parse <EndpointReference> buf(%.*s)", got, buf);
     return 1;
@@ -307,7 +343,7 @@ static int zxid_reg_svc(struct zxid_conf* cf, int bs_reg, int dry_run, const cha
 	      "You may also want to\n"
 	      "  touch %s.all/.bs/%s,%s\n\n", ddimd, path, sha1_name, uiddir, path, sha1_name);
     fflush(stdin);
-    write_all_fd(1, ss->s, ss->len);
+    write_all_fd(fdstdout, ss->s, ss->len);
     zx_str_free(cf->ctx, ss);
     return 0;
   }
@@ -348,13 +384,16 @@ static int zxid_reg_svc(struct zxid_conf* cf, int bs_reg, int dry_run, const cha
   return 0;
 }
 
+/* --------------- addmd --------------- */
+
 /*() Add metadata of a partner to the Circle-of-Trust, represented by the CoT dir */
 
-static int zxid_addmd(struct zxid_conf* cf, char* mdurl, int dry_run, const char* dcot)
+/* Called by:  zxcot_main */
+static int zxid_addmd(zxid_conf* cf, char* mdurl, int dry_run, const char* dcot)
 {
   int got, fd;
   char* p;
-  struct zxid_entity* ent;
+  zxid_entity* ent;
   struct zx_str* ss;
   
   if (mdurl) {
@@ -370,38 +409,57 @@ static int zxid_addmd(struct zxid_conf* cf, char* mdurl, int dry_run, const char
     ERR("***** Parsing metadata failed %d", 0);
     return 1;
   }
+
+  for (; ent; ent = ent->n) {
+    ss = zx_EASY_ENC_SO_md_EntityDescriptor(cf->ctx, ent->ed);
+    if (!ss)
+      return 2;
   
-  ss = zx_EASY_ENC_SO_md_EntityDescriptor(cf->ctx, ent->ed);
-  if (!ss)
-    return 2;
+    if (verbose)
+      fprintf(stderr, "Writing ent(%s) to %s%s\n", ent->eid, dcot, ent->sha1_name);
+    if (dry_run) {
+      write_all_fd(fdstdout, ss->s, ss->len);
+      zx_str_free(cf->ctx, ss);
+      continue;
+    }
   
-  if (verbose)
-    fprintf(stderr, "Writing ent(%.*s) to %s%s\n", ent->eid_len, ent->eid, dcot, ent->sha1_name);
-  if (dry_run) {
-    write_all_fd(1, ss->s, ss->len);
+    fd = open_fd_from_path(O_CREAT | O_WRONLY | O_TRUNC, 0666, "zxcot -a", "%s%s", dcot, ent->sha1_name);
+    if (fd == BADFD) {
+      perror("open metadata for writing metadata to cache");
+      ERR("Failed to open file for writing: sha1_name(%s) to metadata cache", ent->sha1_name);
+      zx_str_free(cf->ctx, ss);
+      return 1;
+    }
+    
+    write_all_fd(fd, ss->s, ss->len);
     zx_str_free(cf->ctx, ss);
-    return 0;
+    close_file(fd, (const char*)__FUNCTION__);
   }
-  
-  fd = open_fd_from_path(O_CREAT | O_WRONLY | O_TRUNC, 0666, "zxcot -a", "%s%s", dcot, ent->sha1_name);
-  if (fd == BADFD) {
-    perror("open metadata for writing metadata to cache");
-    ERR("Failed to open file for writing: sha1_name(%s) to metadata cache", ent->sha1_name);
-    zx_str_free(cf->ctx, ss);
-    return 0;
-  }
-  
-  write_all_fd(fd, ss->s, ss->len);
-  zx_str_free(cf->ctx, ss);
-  close_file(fd, (const char*)__FUNCTION__);
   return 0;
 }
 
+/* --------------- genmd --------------- */
+
+/*() Generate our own metadata */
+
+/* Called by:  zxcot_main */
+static int zxid_genmd(zxid_conf* cf, int dry_run, const char* dcot)
+{
+  zxid_cgi cgi;
+  struct zx_str* meta = zxid_sp_meta(cf, &cgi);
+  memset(&cgi, 0, sizeof(cgi));
+  printf("%.*s", meta->len, meta->s);
+  return 0;
+}
+
+/* --------------- lscot --------------- */
+
 /*() Print a line of Circle-of-Trust listing */
 
-static int zxid_lscot_line(struct zxid_conf* cf, int col_swap, const char* dcot, const char* den)
+/* Called by:  zxid_lscot x2 */
+static int zxid_lscot_line(zxid_conf* cf, int col_swap, const char* dcot, const char* den)
 {
-  struct zxid_entity* ent;
+  zxid_entity* ent;
   char* p;
   int got = read_all(ZXID_MAX_MD, buf, "zxcot line", "%s%s", dcot, den);
   if (!got) {
@@ -414,20 +472,24 @@ static int zxid_lscot_line(struct zxid_conf* cf, int col_swap, const char* dcot,
     ERR("***** Parsing metadata failed for(%s%s)", dcot, den);
     return 2;
   }
-  switch (col_swap) {
-  case 1:  printf("%.*s\t%s%s\n", ent->eid_len, ent->eid, dcot, den); break;
-  case 2:  printf("%.*s\n",       ent->eid_len, ent->eid); break;
-  default: printf("%s%s %.*s\n",  dcot, den, ent->eid_len, ent->eid);
+  while (ent) {
+    switch (col_swap) {
+    case 1:  printf("%-50s %s%s %s\n", ent->eid, dcot, den, STRNULLCHKD(ent->dpy_name)); break;
+    case 2:  printf("%s\n",       ent->eid); break;
+    default: printf("%s%s %-50s %s\n", dcot, den, ent->eid, STRNULLCHKD(ent->dpy_name));
+    }
+    if (strcmp(*den?den:dcot, ent->sha1_name))
+      fprintf(stderr, "Filename(%s) does not match sha1_name(%s)\n", *den?den:dcot, ent->sha1_name);
+    ent = ent->n;
   }
-  if (strcmp(*den?den:dcot, ent->sha1_name))
-    fprintf(stderr, "Filename(%s) does not match sha1_name(%s)\n", *den?den:dcot, ent->sha1_name);
   return 0;
 }
 
 /*() List the contents of the Circle-of-Trust, represented by the CoT directory,
  * in various formats. */
 
-static int zxid_lscot(struct zxid_conf* cf, int col_swap, const char* dcot)
+/* Called by:  zxcot_main */
+static int zxid_lscot(zxid_conf* cf, int col_swap, const char* dcot)
 {
   int got, ret;
   char* p;
@@ -458,6 +520,8 @@ static int zxid_lscot(struct zxid_conf* cf, int col_swap, const char* dcot)
   return 0;
 }
 
+/* ============== MAIN ============ */
+
 #ifndef zxcot_main
 #define zxcot_main main
 #endif
@@ -467,10 +531,9 @@ static int zxid_lscot(struct zxid_conf* cf, int col_swap, const char* dcot)
 /* Called by: */
 int zxcot_main(int argc, char** argv, char** env)
 {
-  struct zxid_conf cf;
-
   strncpy(zx_instance, "\tzxcot", sizeof(zx_instance));
- 
+  cf = zxid_new_conf_to_cf(0);
+
   opt(&argc, &argv, &env);
   
   if (entid) {
@@ -480,16 +543,17 @@ int zxcot_main(int argc, char** argv, char** env)
     printf("%s\n", sha1_name);
     return 0;
   }
-  
-  zxid_init_conf_ctx(&cf, ZXID_PATH);
-  
+    
   if (addmd)
-    return zxid_addmd(&cf, mdurl, dryrun, cotdir);
+    return zxid_addmd(cf, mdurl, dryrun, cotdir);
   
   if (regsvc)
-    return zxid_reg_svc(&cf, regbs, dryrun, dimddir, uiddir);
+    return zxid_reg_svc(cf, regbs, dryrun, dimddir, uiddir);
+
+  if (genmd)
+    return zxid_genmd(cf, dryrun, cotdir);
   
-  return zxid_lscot(&cf, swap, cotdir);
+  return zxid_lscot(cf, swap, cotdir);
 }
 
 /* EOF  --  zxcot.c */
