@@ -1,9 +1,11 @@
-#!/usr/local/bin/perl
-# Copyright (c) 2006 Sampo Kellomaki (sampo@iki.fi), All Rights Reserved.
+#!/usr/bin/perl
+# Copyright (c) 2006-2007 Symlabs (symlabs@symlabs.com), All Rights Reserved.
+# Author: Sampo Kellomaki (sampo@iki.fi)
 # This is confidential unpublished proprietary source code of the author.
 # NO WARRANTY, not even implied warranties. Contains trade secrets.
-# Distribution prohibited unless authorized in writing. See file COPYING.
-# $Id: zxid.pl,v 1.5 2006/09/16 20:00:36 sampo Exp $
+# Distribution prohibited unless authorized in writing.
+# Licensed under Apache License 2.0, see file COPYING.
+# $Id: zxid.pl,v 1.10 2009-08-30 15:09:26 sampo Exp $
 # 31.8.2006, created --Sampo
 
 use Net::SAML;
@@ -11,6 +13,8 @@ use Data::Dumper;
 
 $| = 1;
 
+open STDERR, ">>tmp/zxid.stderr";   # Helps CGI debugging where web server eats the stderr
+
 $cf = Net::SAML::new_conf("/var/zxid/");
 $url = "https://sp1.zxidsp.org:8443/zxid.pl";
 $cdc_url = "https://sp1.zxidcommon.org:8443/zxid.pl"; # NET::SAML::CDC_URL
@@ -45,7 +49,7 @@ if ($op eq 'M') {       # Invoke LECP or redirect to CDC reader.
     &Net::SAML::cdc_read($cf, $cgi);
     exit;
 } elsif ($op eq 'E') {  # Return from CDC read, or start here to by-pass CDC read.
-    exit if Net::SAML::lecp_check($cf, $cgi);
+    #exit if Net::SAML::lecp_check($cf, $cgi);
     exit if Net::SAML::cdc_check($cf, $cgi);
 } elsif ($op eq 'L') {
     warn "Start login";
@@ -77,12 +81,13 @@ if ($op eq 'M') {       # Invoke LECP or redirect to CDC reader.
 	exit if mgmt_screen($cf, $cgi, $ses, $op);
     }
 } elsif ($op eq 'B') {
-    print "CONTENT-TYPE: text/xml\r\n\r\n";
     $md = Net::SAML::sp_meta($cf, $cgi);
-    print $md;
+    printf "CONTENT-LENGTH: %d\r\nCONTENT-TYPE: text/xml\r\n\r\n%s", length $md, $md;
     exit;
+} elsif ($op eq 'K') {
+    warn "Redirect back from SLO";
 } else {
-    die "Unknown op($op)";
+    warn "Unknown op($op)";
 }
 
 print <<HTML;
@@ -180,10 +185,10 @@ sub mgmt_screen {
 	$msg = "SP Initiated logout (SOAP). Session terminated.";
 	return 0;  # Falls thru to login screen.
     } elsif ($op eq 't') {
-	Net::SAML::sp_nireg_redir($cf, $cgi, $ses, 0);
+	Net::SAML::sp_nireg_redir($cf, $cgi, $ses, '');
 	return 1;  # Redirect already happened. Do not show login screen.
     } elsif ($op eq 'u') {
-	Net::SAML::sp_nireg_soap($cf, $cgi, $ses, 0);
+	Net::SAML::sp_nireg_soap($cf, $cgi, $ses, '');
 	$msg = "SP Initiated defederation (SOAP).";
     } elsif ($op eq 'P') {
 	$ret = Net::SAML::sp_dispatch($cf, $cgi, $ses, Net::SAML::zxid_cgi::swig_saml_resp_get($cgi));
