@@ -1,4 +1,5 @@
 /* zxidecp.c  -  Handwritten functions for implementing Enhanced Client Proxy and SP
+ * Copyright (c) 2010 Sampo Kellomaki (sampo@iki.fi), All Rights Reserved.
  * Copyright (c) 2006-2008 Symlabs (symlabs@symlabs.com), All Rights Reserved.
  * Author: Sampo Kellomaki (sampo@iki.fi)
  * This is confidential unpublished proprietary source code of the author.
@@ -10,6 +11,7 @@
  * 12.8.2006, created --Sampo
  * 16.1.2007, split from zxidlib.c --Sampo
  * 7.10.2008, added documentation --Sampo
+ * 14.3.2010, reformed eid presentation --Sampo
  *
  * See: zxid_sp_soap_dispatch() in zxidslo.c for handling PAOS response
  *
@@ -23,6 +25,7 @@
 #include "zxid.h"
 #include "zxidconf.h"
 #include "saml2.h"
+#include "c/zx-paos-data.h"
 
 extern char **environ;
 
@@ -31,7 +34,7 @@ extern char **environ;
  * If you do not know what PAOS, ECP or LECP means, you should read [SAML2bind] specification. */
 
 /* Called by:  zxid_lecp_check */
-struct zx_paos_Request_s* zxid_mk_paos_Request_hdr(struct zxid_conf* cf)
+struct zx_paos_Request_s* zxid_mk_paos_Request_hdr(zxid_conf* cf)
 {
   struct zx_paos_Request_s* hdr= zx_NEW_paos_Request(cf->ctx);
   /*hdr->messageID = zx_ref_str(cf->ctx, "1"); OPTIONAL */
@@ -51,9 +54,9 @@ struct zx_paos_Request_s* zxid_mk_paos_Request_hdr(struct zxid_conf* cf)
  * return:: IdP list data structure or 0 on failure */
 
 /* Called by:  zxid_mk_ecp_Request_hdr */
-struct zx_sp_IDPList_s* zxid_mk_idp_list(struct zxid_conf* cf, char* binding)
+struct zx_sp_IDPList_s* zxid_mk_idp_list(zxid_conf* cf, char* binding)
 {
-  struct zxid_entity* idp;
+  zxid_entity* idp;
   struct zx_md_SingleSignOnService_s* sso_svc;
   struct zx_sp_IDPList_s* idp_list;
   struct zx_sp_IDPEntry_s* idp_entry;
@@ -64,7 +67,7 @@ struct zx_sp_IDPList_s* zxid_mk_idp_list(struct zxid_conf* cf, char* binding)
   
   idp_list = zx_NEW_sp_IDPList(cf->ctx);
   for (; idp; idp = idp->n) {
-    D("IDPList consider idp(%.*s)", idp->eid_len, idp->eid);
+    D("IDPList consider idp(%s)", idp->eid);
     if (!idp->ed->IDPSSODescriptor)
       continue;
     for (sso_svc = idp->ed->IDPSSODescriptor->SingleSignOnService;
@@ -73,13 +76,13 @@ struct zx_sp_IDPList_s* zxid_mk_idp_list(struct zxid_conf* cf, char* binding)
       if (sso_svc->Binding && !memcmp(binding, sso_svc->Binding->s, sso_svc->Binding->len))
 	break;
     if (!sso_svc) {
-      D("Entity(%.*s) does not have any IdP SSO Service with binding(%s)", idp->eid_len, idp->eid, binding);
+      D("Entity(%s) does not have any IdP SSO Service with binding(%s)", idp->eid, binding);
       continue;  /* Not eligible IdP, next one please. */
     }
     
     idp_entry = zx_NEW_sp_IDPEntry(cf->ctx);
-    idp_entry->ProviderID = zx_ref_len_str(cf->ctx, idp->eid_len, idp->eid);
-    /*idp_entry->Name = zx_ref_str(cf->ctx, );*/
+    idp_entry->ProviderID = zx_ref_str(cf->ctx, idp->eid);
+    idp_entry->Name = zx_ref_str(cf->ctx, idp->dpy_name);
     idp_entry->Loc = sso_svc->Location;
     idp_entry->gg.g.n = &idp_list->IDPEntry->gg.g; /* zx_sp_IDPList_PUSH_IDPEntry(idp_list, idp_entry); */
     idp_list->IDPEntry = idp_entry;
@@ -92,7 +95,7 @@ struct zx_sp_IDPList_s* zxid_mk_idp_list(struct zxid_conf* cf, char* binding)
  * If you do not know what PAOS, ECP or LECP means, you should read [SAML2bind] specification. */
 
 /* Called by:  zxid_lecp_check */
-struct zx_ecp_Request_s* zxid_mk_ecp_Request_hdr(struct zxid_conf* cf)
+struct zx_ecp_Request_s* zxid_mk_ecp_Request_hdr(zxid_conf* cf)
 {
   struct zx_ecp_Request_s* hdr= zx_NEW_ecp_Request(cf->ctx);
   hdr->mustUnderstand = zx_ref_str(cf->ctx, ZXID_TRUE);
@@ -116,7 +119,7 @@ struct zx_ecp_Request_s* zxid_mk_ecp_Request_hdr(struct zxid_conf* cf)
  * If you do not know what PAOS, ECP or LECP means, you should read [SAML2bind] specification. */
 
 /* Called by:  main x4, zxid_simple_no_ses_cf x2 */
-struct zx_str* zxid_lecp_check(struct zxid_conf* cf, struct zxid_cgi* cgi)
+struct zx_str* zxid_lecp_check(zxid_conf* cf, zxid_cgi* cgi)
 {
   struct zx_e_Envelope_s* se;
   struct zx_str* env;
