@@ -1,11 +1,11 @@
 /* zxidhlo.c  -  Hello World CGI binary for SAML 2 SP
- * Copyright (c) 2007 Symlabs (symlabs@symlabs.com), All Rights Reserved.
+ * Copyright (c) 2007-2009 Symlabs (symlabs@symlabs.com), All Rights Reserved.
  * Author: Sampo Kellomaki (sampo@iki.fi)
  * This is confidential unpublished proprietary source code of the author.
  * NO WARRANTY, not even implied warranties. Contains trade secrets.
  * Distribution prohibited unless authorized in writing.
  * Licensed under Apache License 2.0, see file COPYING.
- * $Id: zxidhlo.c,v 1.12 2008-09-17 02:39:21 sampo Exp $
+ * $Id: zxidhlo.c,v 1.16 2009-08-30 15:09:26 sampo Exp $
  *
  * 16.1.2007, created --Sampo
  *
@@ -30,7 +30,7 @@
 char* help =
 "zxidhlo  -  SAML 2.0 SP CGI - R" ZXID_REL "\n\
 SAML 2.0 is a standard for federated idenity and Sinlg Sign-On.\n\
-Copyright (c) 2007 Symlabs (symlabs@symlabs.com), All Rights Reserved.\n\
+Copyright (c) 2007-2009 Symlabs (symlabs@symlabs.com), All Rights Reserved.\n\
 Author: Sampo Kellomaki (sampo@iki.fi)\n\
 NO WARRANTY, not even implied warranties. Licensed under Apache License v2.0\n\
 See http://www.apache.org/licenses/LICENSE-2.0\n\
@@ -46,7 +46,8 @@ Usage: zxidhlo [options]   (when used as CGI, no options can be supplied)\n\
 /* CONFIG: You must edit the URL to match your domain name and port */
 
 #define ZXIDHLO "zxidhlo"
-#define CONF "PATH=/var/zxid/&URL=https://sp1.zxidsp.org:8443/" ZXIDHLO "&NOSIG_FATAL=0"
+#define CONF "URL=https://sp1.zxidsp.org:8443/" ZXIDHLO "&NOSIG_FATAL=0&PATH=/var/zxid/"
+//#define CONF "URL=https://lima.tas3.eu:8443/" ZXIDHLO "&NOSIG_FATAL=0&PATH=/var/zxid/"
 
 /* Called by: */
 int main(int argc, char** argv)
@@ -61,12 +62,12 @@ int main(int argc, char** argv)
   /* Helps debugging CGI scripts if you see stderr. */
   /* Reopen stderr only in mini_httpd case */
   p = getenv("SERVER_SOFTWARE");
-  if (p && !memcmp(p, "mini_httpd", sizeof("mini_httpd"))) {
+  if (p && !memcmp(p, "mini_httpd", sizeof("mini_httpd")-1)) {
     close(2);
     if (open("/var/tmp/zxid.stderr", O_WRONLY | O_CREAT | O_APPEND, 0666) != 2)
       exit(2);
   }
-  fprintf(stderr, "=================== Running ===================\n");
+  fprintf(stderr, "=================== Running " ZXIDHLO " ===================\n");
 #endif
 
   if (argc > 1) {
@@ -123,7 +124,8 @@ int main(int argc, char** argv)
     printf("<input type=submit name=gs value=\" Single Logout (SOAP) \">\n");
     printf("<input type=submit name=gt value=\" Defederate (Redir) \">\n");
     printf("<input type=submit name=gu value=\" Defederate (SOAP) \"><br>\n");
-    printf("sid(%s) nid(%s) <a href=\"" ZXIDHLO "?s=%s\">Reload</a>", sid, nid?nid:"?!?", sid);
+    printf("sid(%s) nid(%s) <a href=\"" ZXIDHLO "?s=%s\">Reload</a> | "
+	   "<a href=\"" ZXIDHLO "?o=v&s=%s\">PEP</a>", sid, nid?nid:"?!?", sid, sid);
   }
   
   printf("</form><hr>");
